@@ -1,16 +1,18 @@
 #![forbid(unsafe_code)]
 
+use std::os::unix::ffi::OsStrExt;
 use std::{borrow::Cow, mem};
 
 use crate::common::{SudoPath, SudoString};
 
 pub mod help;
+#[cfg_attr(not(feature = "sudoedit"), allow(unused))]
+pub mod help_edit;
 
 #[cfg(test)]
 mod tests;
 
-// remove dead_code when sudoedit has been implemented
-#[allow(dead_code)]
+#[cfg_attr(not(feature = "sudoedit"), allow(dead_code))]
 pub enum SudoAction {
     Edit(SudoEditOptions),
     Help(SudoHelpOptions),
@@ -22,6 +24,12 @@ pub enum SudoAction {
     Version(SudoVersionOptions),
 }
 
+pub(super) fn is_sudoedit(command_path: Option<String>) -> bool {
+    std::path::Path::new(&command_path.unwrap_or_default())
+        .file_name()
+        .is_some_and(|name| name.as_bytes().starts_with(b"sudoedit"))
+}
+
 impl SudoAction {
     /// try to parse and environment variable assignment
     /// parse command line arguments from the environment and handle errors
@@ -108,8 +116,6 @@ impl TryFrom<SudoOptions> for SudoHelpOptions {
         let help = mem::take(&mut opts.help);
         debug_assert!(help);
 
-        reject_all("--help", opts)?;
-
         Ok(Self {})
     }
 }
@@ -125,8 +131,6 @@ impl TryFrom<SudoOptions> for SudoVersionOptions {
         let version = mem::take(&mut opts.version);
         debug_assert!(version);
 
-        reject_all("--version", opts)?;
-
         Ok(Self {})
     }
 }
@@ -218,7 +222,7 @@ impl TryFrom<SudoOptions> for SudoValidateOptions {
 }
 
 // sudo -e [-ABkNnS] [-r role] [-t type] [-C num] [-D directory] [-g group] [-h host] [-p prompt] [-R directory] [-T timeout] [-u user] file ...
-#[allow(dead_code)]
+#[cfg_attr(not(feature = "sudoedit"), allow(dead_code))]
 pub struct SudoEditOptions {
     // -B
     pub bell: bool,
@@ -355,7 +359,7 @@ pub struct SudoRunOptions {
     // -B
     pub bell: bool,
     // -E
-    pub preserve_env: PreserveEnv,
+    /* ignored, part of env_var_list */
     // -k
     pub reset_timestamp: bool,
     // -n
@@ -384,7 +388,6 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
 
     fn try_from(mut opts: SudoOptions) -> Result<Self, Self::Error> {
         let bell = mem::take(&mut opts.bell);
-        let preserve_env = mem::take(&mut opts.preserve_env);
         let reset_timestamp = mem::take(&mut opts.reset_timestamp);
         let non_interactive = mem::take(&mut opts.non_interactive);
         let stdin = mem::take(&mut opts.stdin);
@@ -424,7 +427,6 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
 
         Ok(Self {
             bell,
-            preserve_env,
             reset_timestamp,
             non_interactive,
             stdin,
@@ -455,7 +457,7 @@ struct SudoOptions {
     // -U
     other_user: Option<SudoString>,
     // -E
-    preserve_env: PreserveEnv,
+    /* ignored, part of env_var_list */
     // -s
     shell: bool,
     // -S
@@ -484,33 +486,10 @@ struct SudoOptions {
     // -V
     version: bool,
 
-    // arguments passed straight through, either seperated by -- or just trailing.
+    // arguments passed straight through, either separated by -- or just trailing.
     positional_args: Vec<String>,
 }
 
-#[derive(Default, Debug, Clone, PartialEq)]
-pub enum PreserveEnv {
-    #[default]
-    Nothing,
-    Everything,
-    Only(Vec<String>),
-}
-
-impl PreserveEnv {
-    #[cfg(test)]
-    pub fn try_into_only(self) -> Result<Vec<String>, Self> {
-        if let Self::Only(v) = self {
-            Ok(v)
-        } else {
-            Err(self)
-        }
-    }
-
-    pub fn is_nothing(&self) -> bool {
-        matches!(self, Self::Nothing)
-    }
-}
-
 #[derive(Debug, Clone, PartialEq)]
 pub enum List {
     Once,
@@ -544,12 +523,11 @@ impl SudoArg {
     ];
 
     /// argument assignments and shorthand options preprocessing
-    fn normalize_arguments<I>(iter: I) -> Result<Vec<Self>, String>
+    /// the iterator should only iterate over the actual arguments
+    fn normalize_arguments<I>(mut arg_iter: I) -> Result<Vec<Self>, String>
     where
-        I: IntoIterator<Item = String>,
+        I: Iterator<Item = String>,
     {
-        // the first argument is the sudo command - so we can skip it
-        let mut arg_iter = iter.into_iter().skip(1);
         let mut processed = vec![];
 
         while let Some(arg) = arg_iter.next() {
@@ -563,14 +541,14 @@ impl SudoArg {
                     // only accept arguments when one is expected
                     // `--preserve-env` is special as it only takes an argument using this `key=value` syntax
                     if !Self::TAKES_ARGUMENT.contains(&key) && key != "preserve-env" {
-                        Err(format!("'{}' does not take any arguments", key))?;
+                        Err(format!("'{key}' does not take any arguments"))?;
                     }
                     processed.push(SudoArg::Argument("--".to_string() + key, value.to_string()));
                 } else if Self::TAKES_ARGUMENT.contains(&unprefixed) {
                     if let Some(next) = arg_iter.next() {
                         processed.push(SudoArg::Argument(arg, next));
                     } else {
-                        Err(format!("'{}' expects an argument", &arg))?;
+                        Err(format!("'{arg}' expects an argument"))?;
                     }
                 } else {
                     processed.push(SudoArg::Flag(arg));
@@ -598,7 +576,7 @@ impl SudoArg {
                             // short version of --help has no arguments
                             processed.push(SudoArg::Flag(flag));
                         } else {
-                            Err(format!("'-{}' expects an argument", curr))?;
+                            Err(format!("'-{curr}' expects an argument"))?;
                         }
                         break;
                     } else {
@@ -654,8 +632,16 @@ impl SudoOptions {
         I: IntoIterator<Item = T>,
         T: Into<String> + Clone,
     {
-        let mut options = Self::default();
-        let arg_iter = SudoArg::normalize_arguments(iter.into_iter().map(Into::into))?
+        let mut arg_iter = iter.into_iter().map(Into::into);
+
+        let invoked_as_sudoedit = is_sudoedit(arg_iter.next());
+
+        let mut options = Self {
+            edit: invoked_as_sudoedit,
+            ..Self::default()
+        };
+
+        let arg_iter = SudoArg::normalize_arguments(arg_iter)?
             .into_iter()
             .peekable();
 
@@ -666,9 +652,11 @@ impl SudoOptions {
                         options.bell = true;
                     }
                     "-E" | "--preserve-env" => {
-                        options.preserve_env = PreserveEnv::Everything;
+                        eprintln_ignore_io_error!(
+                            "warning: preserving the entire environment is not supported, `{flag}` is ignored"
+                        )
                     }
-                    "-e" | "--edit" => {
+                    "-e" | "--edit" if !invoked_as_sudoedit => {
                         options.edit = true;
                     }
                     "-H" | "--set-home" => {
@@ -716,15 +704,13 @@ impl SudoOptions {
                         options.chdir = Some(SudoPath::from_cli_string(value));
                     }
                     "-E" | "--preserve-env" => {
-                        let split_value = || value.split(',').map(str::to_string);
-                        match &mut options.preserve_env {
-                            PreserveEnv::Nothing => {
-                                options.preserve_env = PreserveEnv::Only(split_value().collect())
-                            }
-                            PreserveEnv::Everything => {}
-                            PreserveEnv::Only(list) => list.extend(split_value()),
-                        }
-                        // options.preserve_env = value.split(',').map(str::to_string).collect()
+                        options
+                            .env_var_list
+                            .extend(value.split(',').filter_map(|var| {
+                                std::env::var(var)
+                                    .ok()
+                                    .map(|value| (var.to_string(), value))
+                            }));
                     }
                     "-g" | "--group" => {
                         options.group = Some(SudoString::from_cli_string(value));
@@ -745,7 +731,24 @@ impl SudoOptions {
                 SudoArg::Environment(key, value) => {
                     options.env_var_list.push((key, value));
                 }
-                SudoArg::Rest(rest) => {
+                SudoArg::Rest(mut rest) => {
+                    if let Some(cmd) = rest.first() {
+                        let cmd = std::path::Path::new(cmd);
+                        // This checks if the last character in the path is a /. This
+                        // works because the OS directly splits at b'/' without regards
+                        // for if it is part of another character (which it can't be
+                        // with UTF-8 anyways).
+                        let is_dir = cmd.as_os_str().as_bytes().ends_with(b"/");
+                        if !options.edit
+                            && !is_dir
+                            && (cmd.ends_with("sudoedit") || cmd.ends_with("sudoedit-rs"))
+                        {
+                            eprintln_ignore_io_error!("sudoedit doesn't need to be run via sudo");
+                            options.edit = true;
+                            rest.remove(0);
+                        }
+                    }
+
                     options.positional_args = rest;
                 }
             }
@@ -787,12 +790,6 @@ impl<T> IsAbsent for Vec<T> {
     }
 }
 
-impl IsAbsent for PreserveEnv {
-    fn is_absent(&self) -> bool {
-        self.is_nothing()
-    }
-}
-
 fn ensure_is_absent(context: &str, thing: &dyn IsAbsent, name: &str) -> Result<(), String> {
     if thing.is_absent() {
         Ok(())
@@ -822,7 +819,6 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
         login,
         non_interactive,
         other_user,
-        preserve_env,
         shell,
         stdin,
         prompt,
@@ -848,7 +844,6 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
         tuple!(login),
         tuple!(non_interactive),
         tuple!(other_user),
-        tuple!(preserve_env),
         tuple!(remove_timestamp),
         tuple!(reset_timestamp),
         tuple!(shell),
